adjustedProfileLik <- function(dispersion, y, design, offset, adjust=TRUE)
# tagwise Cox-Reid adjusted profile likelihoods for the dispersion
# dispersion can be scalar or tagwise vector
# y is matrix: rows are genes/tags/transcripts, columns are samples/libraries
# offset is matrix of the same dimensions as y
# Yunshun Chen, Gordon Smyth, Aaron Lun
# Created June 2010. Last modified 3 Nov 2013.
{
	if(any(dim(y)!=dim(offset))) offset <- expandAsMatrix(offset,dim(y))
	ntags <- nrow(y)
	nlibs <- ncol(y)
	if(length(dispersion)==1) dispersion <- rep(dispersion,ntags)

#	Fit tagwise linear models. This is actually the most time-consuming
#	operation that I can see for this function.
	fit <- glmFit(y,design=design,dispersion=dispersion,offset=offset,prior.count.total=0)

#	Compute log-likelihood.
	mu <- fit$fitted
	if(dispersion[1] == 0){
		loglik <- rowSums(dpois(y,lambda=mu,log = TRUE))
	} else {
		loglik <- rowSums(dnbinom(y,size=1/dispersion,mu=mu,log = TRUE))
	}
	if (!adjust) {
		return(loglik)
	}
		
#	Calculating the Cox-Reid adjustment.
	if(ncol(design)==1) {
		D <- rowSums(mu/(1+mu*dispersion))
		cr <- 0.5*log(abs(D))
	} else {
		W <- mu/(1+dispersion*mu)
		cr<-.Call("cr_adjustment", W, design, nrow(design), PACKAGE="edgeR")
	}
 
	return(loglik - cr)
}

